# Copyright 2019 Akiomi Kamakura
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pilgram._1977 import _1977
from pilgram._version import __version__
from pilgram.aden import aden
from pilgram.brannan import brannan
from pilgram.brooklyn import brooklyn
from pilgram.clarendon import clarendon
from pilgram.earlybird import earlybird
from pilgram.gingham import gingham
from pilgram.hudson import hudson
from pilgram.inkwell import inkwell
from pilgram.kelvin import kelvin
from pilgram.lark import lark
from pilgram.lofi import lofi
from pilgram.maven import maven
from pilgram.mayfair import mayfair
from pilgram.moon import moon
from pilgram.nashville import nashville
from pilgram.perpetua import perpetua
from pilgram.reyes import reyes
from pilgram.rise import rise
from pilgram.slumber import slumber
from pilgram.stinson import stinson
from pilgram.toaster import toaster
from pilgram.valencia import valencia
from pilgram.walden import walden
from pilgram.willow import willow
from pilgram.xpro2 import xpro2

__all__ = [
    "__version__",
    "_1977",
    "aden",
    "brannan",
    "brooklyn",
    "clarendon",
    "earlybird",
    "gingham",
    "hudson",
    "inkwell",
    "kelvin",
    "lark",
    "lofi",
    "maven",
    "mayfair",
    "moon",
    "nashville",
    "perpetua",
    "reyes",
    "rise",
    "slumber",
    "stinson",
    "toaster",
    "valencia",
    "walden",
    "willow",
    "xpro2",
]
